@@ -1,7 +1,8 @@
 # debian/changelog -- lintian check script -*- perl -*-
 
-# Copyright © 1998 Christian Schwarz
-# Copyright © 2019-2020 Chris Lamb <lamby@debian.org>
+# Copyright (C) 1998 Christian Schwarz
+# Copyright (C) 2019-2020 Chris Lamb <lamby@debian.org>
+# Copyright (C) 2021 Felix Lechner
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, you can find it on the World Wide
-# Web at http://www.gnu.org/copyleft/gpl.html, or write to the Free
+# Web at https://www.gnu.org/copyleft/gpl.html, or write to the Free
 # Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 # MA 02110-1301, USA.
 
@@ -35,27 +36,36 @@ use Path::Tiny;
 use Syntax::Keyword::Try;
 use Unicode::UTF8 qw(valid_utf8 decode_utf8 encode_utf8);
 
-use Lintian::Inspect::Changelog;
-use Lintian::Inspect::Changelog::Version;
+use Lintian::Changelog;
+use Lintian::Changelog::Version;
 use Lintian::IPC::Run3 qw(safe_qx);
 use Lintian::Relation::Version qw(versions_gt);
 use Lintian::Spelling qw(check_spelling);
 
+const my $EMPTY => q{};
+const my $DOUBLE_QUOTE => q{"};
+const my $GREATER_THAN => q{>};
+const my $APPROXIMATELY_EQUAL => q{~};
+
+const my $NOT_EQUALS => q{!=};
+const my $ARROW => q{->};
+
+const my $MAXIMUM_WIDTH => 82;
+const my $FIRST_ARCHIVED_BUG_NUMBER => 50_004;
+const my $OUT_OF_REACH_BUG_NUMBER => 1_500_000;
+
 use Moo;
 use namespace::clean;
 
 with 'Lintian::Check';
 
-const my $EMPTY => q{};
-
-const my $MAXIMUM_WIDTH => 82;
-const my $FIRST_ARCHIVED_BUG_NUMBER => 50_004;
-const my $OUT_OF_REACH_BUG_NUMBER => 1_500_000;
-
 sub spelling_tag_emitter {
-    my ($self, @orig_args) = @_;
+    my ($self, $tag_name, $item, @orig_args) = @_;
+
+    my $pointer = $item->pointer($.);
+
     return sub {
-        return $self->hint(@orig_args, @_);
+        return $self->pointed_hint($tag_name, $pointer, @orig_args, @_);
     };
 }
 
@@ -76,27 +86,31 @@ sub source {
 
     my $latest_entry = $entries[0];
 
+    my $changelog_item = $self->processable->changelog_item;
+    my $latest_pointer = $changelog_item->pointer($latest_entry->position);
+
     my $changes = $group->changes;
     if ($changes) {
         my $contents = path($changes->path)->slurp;
         # make sure dot matches newlines, as well
         if ($contents =~ qr/BEGIN PGP SIGNATURE.*END PGP SIGNATURE/ms) {
 
-            $self->hint('unreleased-changelog-distribution')
+            $self->pointed_hint('unreleased-changelog-distribution',
+                $latest_pointer)
               if $latest_entry->Distribution eq 'UNRELEASED';
         }
     }
 
     my $versionstring = $processable->fields->value('Version');
-    my $latest_version = Lintian::Inspect::Changelog::Version->new;
+    my $latest_version = Lintian::Changelog::Version->new;
 
     try {
         $latest_version->assign($versionstring, $processable->native);
 
     } catch {
         my $indicator= ($processable->native ? $EMPTY : 'non-') . 'native';
-        $self->hint('malformed-debian-changelog-version',
-            $versionstring, "(for $indicator)");
+        $self->pointed_hint('malformed-debian-changelog-version',
+            $latest_pointer,$versionstring, "(for $indicator)");
         undef $latest_version;
 
         # perlcritic 1.140-1 requires a semicolon on the next line
@@ -104,8 +118,9 @@ sub source {
 
     if (defined $latest_version) {
 
-        $self->hint('hyphen-in-upstream-part-of-debian-changelog-version',
-            $latest_version->upstream)
+        $self->pointed_hint(
+            'hyphen-in-upstream-part-of-debian-changelog-version',
+            $latest_pointer,$latest_version->upstream)
           if !$processable->native && $latest_version->upstream =~ qr/-/;
 
         # unstable, testing, and stable shouldn't be used in Debian
@@ -119,8 +134,8 @@ sub source {
             my $revision = $latest_version->maintainer_revision;
             my $distribution = $latest_entry->Distribution;
 
-            $self->hint('version-refers-to-distribution',
-                $latest_version->literal)
+            $self->pointed_hint('version-refers-to-distribution',
+                $latest_pointer,$latest_version->literal)
               if ($revision =~ /testing|(?:un)?stable/i)
               || (
                 ($distribution eq 'unstable'|| $distribution eq 'experimental')
@@ -147,9 +162,13 @@ sub source {
 
             my $suggestion = "$expected~$candidate_string$increment_string";
 
-            $self->hint(
+            $self->pointed_hint(
                 'rc-version-greater-than-expected-version',
-                $examine, '>',$expected, "(consider using $suggestion)",
+                $latest_pointer,
+                $examine,
+                $GREATER_THAN,
+                $expected,
+                "(consider using $suggestion)",
               )
               if $latest_version->maintainer_revision eq '1'
               || $latest_version->maintainer_revision=~ /^0(?:\.1|ubuntu1)?$/
@@ -163,28 +182,34 @@ sub source {
         my $latest_timestamp = $latest_entry->Timestamp;
         my $previous_timestamp = $previous_entry->Timestamp;
 
-        my $previous_version = Lintian::Inspect::Changelog::Version->new;
+        my $previous_version = Lintian::Changelog::Version->new;
         try {
             $previous_version->assign($previous_entry->Version,
                 $processable->native);
         } catch {
             my $indicator= ($processable->native ? $EMPTY : 'non-') . 'native';
-            $self->hint(
+            $self->pointed_hint(
                 'odd-historical-debian-changelog-version',
-                $previous_entry->Version . " (for $indicator)"
+                $changelog_item->pointer($previous_entry->position),
+                $previous_entry->Version,
+                "(for $indicator)"
             );
             undef $previous_version;
         }
 
         if ($latest_timestamp && $previous_timestamp) {
 
-            $self->hint('latest-debian-changelog-entry-without-new-date')
+            $self->pointed_hint(
+                'latest-debian-changelog-entry-without-new-date',
+                $latest_pointer)
               if $latest_timestamp <= $previous_timestamp
               && lc($latest_entry->Distribution) ne 'unreleased';
         }
 
         if (defined $latest_version) {
-            foreach my $entry (@entries[1..$#entries]) {
+
+            # skip first
+            for my $entry (@entries[1..$#entries]) {
 
                 # cannot use parser; nativeness may differ
                 my ($no_epoch) = ($entry->Version =~ qr/^(?:[^:]+:)?([^:]+)$/);
@@ -195,13 +220,16 @@ sub source {
                 # disallowed even if epochs differ; see tag description
                 if (   $latest_version->no_epoch eq $no_epoch
                     && $latest_entry->Source eq $entry->Source) {
-                    $self->hint(
+
+                    $self->pointed_hint(
 'latest-debian-changelog-entry-reuses-existing-version',
-                        $latest_version->literal. ' ~= '
-                          . $entry->Version
-                          . ' (last used: '
-                          . $entry->Date . ')'
+                        $latest_pointer,
+                        $latest_version->literal,
+                        $APPROXIMATELY_EQUAL,
+                        $entry->Version,
+                        '(last used: '. $entry->Date . ')'
                     );
+
                     last;
                 }
             }
@@ -229,13 +257,15 @@ sub source {
                   && $latest_version->maintainer_revision ne '0';
             }
 
-            $self->hint('changelog-file-missing-explicit-entry',
-                    $previous_version->literal
-                  . " -> $expected_previous (missing) -> "
-                  . $latest_version->literal)
+            $self->pointed_hint(
+                'changelog-file-missing-explicit-entry',$latest_pointer,
+                $previous_version->literal, $ARROW,
+                "$expected_previous (missing)", $ARROW,
+                $latest_version->literal
+              )
               unless $previous_version->literal eq $expected_previous
-              || $latest_entry->Distribution eq 'buster'
-              || $previous_entry->Distribution eq 'buster'
+              || $latest_entry->Distribution eq 'bullseye'
+              || $previous_entry->Distribution eq 'bullseye'
               || $latest_entry->Distribution =~ /-security$/i;
 
             if (   $latest_version->epoch eq $previous_version->epoch
@@ -243,8 +273,9 @@ sub source {
                 && $latest_entry->Source eq $previous_entry->Source
                 && !$processable->native) {
 
-                $self->hint(
-                    'possible-new-upstream-release-without-new-version')
+                $self->pointed_hint(
+                    'possible-new-upstream-release-without-new-version',
+                    $latest_pointer)
                   if $latest_entry->Changes
                   =~ /^\s*\*\s+new\s+upstream\s+(?:\S+\s+)?release\b/im;
 
@@ -270,22 +301,24 @@ sub source {
                   && !length $previous_version->source_nmu
                   && $latest_version->source_nmu != 1;
 
-                $self->hint('non-consecutive-debian-revision',
-                        $previous_version->literal . ' -> '
-                      . $latest_version->literal)
-                  if $non_consecutive;
+                $self->pointed_hint(
+                    'non-consecutive-debian-revision',
+                    $latest_pointer,$previous_version->literal,
+                    $ARROW,$latest_version->literal
+                )if $non_consecutive;
             }
 
             if ($latest_version->epoch ne $previous_version->epoch) {
-                $self->hint('epoch-change-without-comment',
-                        $previous_version->literal . ' -> '
-                      . $latest_version->literal)
-                  unless $latest_entry->Changes =~ /\bepoch\b/im;
+                $self->pointed_hint(
+                    'epoch-change-without-comment',$latest_pointer,
+                    $previous_version->literal, $ARROW,
+                    $latest_version->literal
+                )unless $latest_entry->Changes =~ /\bepoch\b/im;
 
-                $self->hint(
+                $self->pointed_hint(
                     'epoch-changed-but-upstream-version-did-not-go-backwards',
-                    $previous_version->literal . ' -> '
-                      . $latest_version->literal
+                    $latest_pointer,$previous_version->literal,
+                    $ARROW,$latest_version->literal
                   )
                   unless $processable->native
                   || versions_gt($previous_version->upstream,
@@ -322,10 +355,10 @@ sub binary {
     my @news_files
       = grep { $_->basename =~ m{\A NEWS\.Debian (?:\.gz)? \Z}ixsm }@doc_files;
 
-    $self->hint('debian-news-file-not-compressed', $_->name)
+    $self->pointed_hint('debian-news-file-not-compressed', $_->pointer)
       for grep { $_->basename !~ m{\.gz$} } @news_files;
 
-    $self->hint('wrong-name-for-debian-news-file', $_->name)
+    $self->pointed_hint('wrong-name-for-debian-news-file', $_->pointer)
       for grep { $_->basename =~ m{\.gz$} && $_->basename ne 'NEWS.Debian.gz' }
       @news_files;
 
@@ -334,10 +367,10 @@ sub binary {
     } @doc_files;
 
     # ubuntu permits symlinks; their profile suppresses the tag
-    $self->hint('debian-changelog-file-is-a-symlink', $_->name)
+    $self->pointed_hint('debian-changelog-file-is-a-symlink', $_->pointer)
       for grep { $_->is_symlink } @changelog_files;
 
-    $self->hint('changelog-file-not-compressed', $_->name)
+    $self->pointed_hint('changelog-file-not-compressed', $_->pointer)
       for grep { $_->basename !~ m{ \.gz \Z}xsm } @changelog_files;
 
     # Check if changelog files are compressed with gzip -9.
@@ -345,77 +378,87 @@ sub binary {
     # with a file named ChangeLog.  If there's also a
     # changelog file, it might be a duplicate, or the packager
     # may have installed NEWS as changelog intentionally.
-    for my $path (@changelog_files) {
+    for my $item (@changelog_files) {
 
         next
-          unless $path->basename =~ m{ \.gz \Z}xsm;
+          unless $item->basename =~ m{ \.gz \Z}xsm;
 
-        my $resolved = $path->resolve_path;
+        my $resolved = $item->resolve_path;
         next
           unless defined $resolved;
 
-        $self->hint('changelog-not-compressed-with-max-compression',
-            $path->name)
-          unless $resolved->file_info =~ /max compression/;
+        $self->pointed_hint('changelog-not-compressed-with-max-compression',
+            $item->pointer)
+          unless $resolved->file_type =~ /max compression/;
     }
 
-    my $found_html = 0;
-    $found_html = 1
-      if any { $_->basename =~ /^changelog\.html(?:\.gz)?$/ } @changelog_files;
+    my @html_changelogs
+      = grep { $_->basename =~ /^changelog\.html(?:\.gz)?$/ } @changelog_files;
+    my @text_changelogs
+      = grep { $_->basename =~ /^changelog(?:\.gz)?$/ } @changelog_files;
 
-    my $found_text = 0;
-    $found_text = 1
-      if any { $_->basename =~ /^changelog(?:\.gz)?$/ } @changelog_files;
+    if (!@text_changelogs) {
+
+        $self->pointed_hint('html-changelog-without-text-version', $_->pointer)
+          for @html_changelogs;
+    }
 
     my $packagepath = 'usr/share/doc/' . $self->processable->name;
-    my $packagenewspath
+    my $news_item
       = $self->processable->installed->resolve_path(
         "$packagepath/NEWS.Debian.gz");
 
     my $news;
-    if (defined $packagenewspath && $packagenewspath->is_file) {
+    if (defined $news_item && $news_item->is_file) {
 
-        my $bytes = safe_qx('gunzip', '-c', $packagenewspath->unpacked_path);
+        my $bytes = safe_qx('gunzip', '-c', $news_item->unpacked_path);
 
         # another check complains about invalid encoding
         if (valid_utf8($bytes)) {
 
             my $contents = decode_utf8($bytes);
-            my $changelog = Lintian::Inspect::Changelog->new;
-            $changelog->parse($contents);
+            my $newslog = Lintian::Changelog->new;
+            $newslog->parse($contents);
 
-            if (my @errors = @{$changelog->errors}) {
-                for (@errors) {
-                    $self->hint('syntax-error-in-debian-news-file',
-                        "line $_->[0]","\"$_->[1]\"");
-                }
+            for my $error (@{$newslog->errors}) {
+
+                my $position = $error->[0];
+                my $condition = $error->[1];
+
+                my $pointer = $news_item->pointer($position);
+
+                $self->pointed_hint('syntax-error-in-debian-news-file',
+                    $pointer, $DOUBLE_QUOTE . $condition . $DOUBLE_QUOTE);
             }
 
             # Some checks on the most recent entry.
-            if ($changelog->entries && defined @{$changelog->entries}[0]) {
-                ($news) = @{$changelog->entries};
-                if ($news->Distribution && $news->Distribution eq 'UNRELEASED')
-                {
-                    $self->hint('debian-news-entry-has-strange-distribution',
-                        $news->Distribution);
-                }
+            if ($newslog->entries && defined @{$newslog->entries}[0]) {
+
+                $news = @{$newslog->entries}[0];
+
+                my $pointer = $news_item->pointer($news->position);
+
+                $self->pointed_hint(
+                    'debian-news-entry-has-strange-distribution',
+                    $pointer,$news->Distribution)
+                  if length $news->Distribution
+                  && $news->Distribution eq 'UNRELEASED';
+
                 check_spelling(
-                    $self->profile,
+                    $self->data,
                     $news->Changes,
                     $group->spelling_exceptions,
                     $self->spelling_tag_emitter(
-                        'spelling-error-in-news-debian'));
-                if ($news->Changes =~ /^\s*\*\s/) {
-                    $self->hint('debian-news-entry-uses-asterisk');
-                }
+                        'spelling-error-in-news-debian', $news_item
+                    )
+                );
+
+                $self->pointed_hint('debian-news-entry-uses-asterisk',$pointer)
+                  if $news->Changes =~ /^ \s* [*] \s /x;
             }
         }
     }
 
-    if ($found_html && !$found_text) {
-        $self->hint('html-changelog-without-text-version');
-    }
-
     # is this a native Debian package?
     # If the version is missing, we assume it to be non-native
     # as it is the most likely case.
@@ -442,14 +485,12 @@ sub binary {
         # native Debian package
         if (any { m/^changelog(?:\.gz)?$/} map { $_->basename } @doc_files) {
             # everything is fine
-        } elsif (
-            my $chg = first {
-                m/^changelog\.debian(?:\.gz)$/i;
-            }
-            map { $_->basename } @doc_files
-        ) {
-            $self->hint('wrong-name-for-changelog-of-native-package',
-                "usr/share/doc/$pkg/$chg");
+        } elsif (my $chg
+            = first {$_->basename =~ m/^changelog[.]debian(?:\.gz)$/i;}
+            @doc_files) {
+            $self->pointed_hint('wrong-name-for-changelog-of-native-package',
+                $chg->pointer);
+
         } else {
             $self->hint(
                 'no-changelog',
@@ -470,10 +511,13 @@ sub binary {
             # everything is fine
         } else {
             # search for changelogs with wrong file name
-            for (map { $_->basename } @doc_files) {
-                if (m/^change/i and not m/debian/i) {
-                    $self->hint('wrong-name-for-upstream-changelog',
-                        "usr/share/doc/$pkg/$_");
+            for my $item (@doc_files) {
+
+                if (   $item->basename =~ m/^change/i
+                    && $item->basename !~ m/debian/i) {
+
+                    $self->pointed_hint('wrong-name-for-upstream-changelog',
+                        $item->pointer);
                     last;
                 }
             }
@@ -485,17 +529,16 @@ sub binary {
             map { $_->basename } @doc_files
         ) {
             # everything is fine
-        } elsif (
-            my $chg = first {
-                m/^changelog\.debian(?:\.gz)?$/i;
-            }
-            map { $_->basename } @doc_files
-        ) {
-            $self->hint('wrong-name-for-debian-changelog-file',
-                "usr/share/doc/$pkg/$chg");
+        } elsif (my $chg
+            = first {$_->basename =~ m/^changelog\.debian(?:\.gz)?$/i;}
+            @doc_files) {
+            $self->pointed_hint('wrong-name-for-debian-changelog-file',
+                $chg->pointer);
+
         } else {
             if ($foreign_pkg && $found_upstream_text_changelog) {
                 $self->hint('debian-changelog-file-missing-or-wrong-name');
+
             } elsif ($foreign_pkg) {
                 $self->hint(
                     'no-changelog',
@@ -512,33 +555,34 @@ sub binary {
         }
     }
 
-    my $dchpath = $self->processable->changelog_path;
+    my $changelog_item = $self->processable->changelog_item;
     return
-      unless length $dchpath;
+      unless defined $changelog_item;
 
     # another check complains about invalid encoding
-    my $bytes = path($dchpath)->slurp;
-    return
-      unless valid_utf8($bytes);
-
     my $changelog = $processable->changelog;
-    if (my @errors = @{$changelog->errors}) {
-        foreach (@errors) {
-            $self->hint('syntax-error-in-debian-changelog',
-                "line $_->[0]","\"$_->[1]\"");
-        }
+
+    for my $error (@{$changelog->errors}) {
+
+        my $position = $error->[0];
+        my $condition = $error->[1];
+
+        my $pointer = $changelog_item->pointer($position);
+
+        $self->pointed_hint('syntax-error-in-debian-changelog',
+            $pointer, $DOUBLE_QUOTE . $condition . $DOUBLE_QUOTE);
     }
 
     # Check for some things in the raw changelog file and compute the
     # "offset" to the first line of the first entry.  We use this to
     # report the line number of "too-long" lines.  (#657402)
-    my $chloff = $self->check_dch($dchpath);
+    my $real_start = $self->check_dch($changelog_item);
 
     my @entries = @{$changelog->entries};
 
     # all versions from the changelog
     my %allversions
-      = map { $_ => 1 } grep { defined $_ } map { $_->Version } @entries;
+      = map { $_ => 1 } grep { defined } map { $_->Version } @entries;
 
     # checks applying to all entries
     for my $entry (@entries) {
@@ -546,37 +590,32 @@ sub binary {
         my $position = $entry->position;
         my $version = $entry->Version;
 
+        my $pointer = $changelog_item->pointer($position);
+
         if (length $entry->Maintainer) {
             my ($parsed) = Email::Address::XS->parse($entry->Maintainer);
 
             unless ($parsed->is_valid) {
-                $self->hint(
+
+                $self->pointed_hint(
                     'bogus-mail-host-in-debian-changelog',
-                    $entry->Maintainer,
-                    "version $version",
-                    "(line $position)"
+                    $pointer,$entry->Maintainer,"(for version $version)",
                 );
                 next;
             }
 
-            unless (
-                all { length }
-                ($parsed->address, $parsed->user, $parsed->host)
-            ) {
-                $self->hint(
+            unless (all { length }
+                ($parsed->address, $parsed->user, $parsed->host)) {
+                $self->pointed_hint(
                     'bogus-mail-host-in-debian-changelog',
-                    $parsed->format,
-                    "version $version",
-                    "(line $position)"
+                    $pointer,$parsed->format,"(for version $version)",
                 );
                 next;
             }
 
-            $self->hint(
+            $self->pointed_hint(
                 'bogus-mail-host-in-debian-changelog',
-                $parsed->address,
-                "version $version",
-                "(line $position)"
+                $pointer, $parsed->address,"(for version $version)",
               )
               unless is_domain($parsed->host,
                 {domain_disable_tld_validation => 1});
@@ -584,27 +623,35 @@ sub binary {
     }
 
     my $INVALID_DATES
-      = $self->profile->load_data('changelog-file/invalid-dates',
-        qr/\s*=\>\s*/);
+      = $self->data->load('changelog-file/invalid-dates',qr/\s*=\>\s*/);
 
     if (@entries) {
 
         # checks related to the latest entry
         my $latest_entry = $entries[0];
 
+        my $latest_pointer = $changelog_item->pointer($latest_entry->position);
+
         my $latest_timestamp = $latest_entry->Timestamp;
 
         if ($latest_timestamp) {
+
             my $warned = 0;
             my $longdate = $latest_entry->Date;
-            foreach my $re ($INVALID_DATES->all()) {
+
+            for my $re ($INVALID_DATES->all()) {
                 if ($longdate =~ m/($re)/i) {
+
+                    my $match = $1;
                     my $repl = $INVALID_DATES->value($re);
-                    $self->hint('invalid-date-in-debian-changelog',
-                        "($1 -> $repl)");
+
+                    $self->pointed_hint('invalid-date-in-debian-changelog',
+                        $latest_pointer,"($match", $ARROW, "$repl)");
+
                     $warned = 1;
                 }
             }
+
             my ($weekday_declared, $numberportion)
               = split(m/,\s*/, $longdate, 2);
             $numberportion //= $EMPTY;
@@ -614,11 +661,12 @@ sub binary {
                 $tz = $1;
                 $weekday_actual = time2str('%a', $latest_timestamp, $tz);
             }
+
             if (not $warned and $tz and $weekday_declared ne $weekday_actual) {
                 my $real_weekday = time2str('%A', $latest_timestamp, $tz);
                 my $short_date = time2str('%Y-%m-%d', $latest_timestamp, $tz);
-                $self->hint('debian-changelog-has-wrong-day-of-week',
-                    "$short_date is a $real_weekday");
+                $self->pointed_hint('debian-changelog-has-wrong-day-of-week',
+                    $latest_pointer,"$short_date was a $real_weekday");
             }
         }
 
@@ -629,22 +677,22 @@ sub binary {
 
             my $previous_timestamp = $previous_entry->Timestamp;
 
-            if ($latest_timestamp && $previous_timestamp) {
-
-                $self->hint('latest-changelog-entry-without-new-date')
-                  if $latest_timestamp <= $previous_timestamp
-                  && $latest_entry->Distribution ne 'UNRELEASED';
-            }
+            $self->pointed_hint('latest-changelog-entry-without-new-date',
+                $latest_pointer)
+              if defined $latest_timestamp
+              && defined $previous_timestamp
+              && $latest_timestamp <= $previous_timestamp
+              && $latest_entry->Distribution ne 'UNRELEASED';
 
             my $latest_dist = lc $latest_entry->Distribution;
             my $previous_dist = lc $previous_entry->Distribution;
-            if (    $latest_dist eq 'unstable'
-                and $previous_dist eq 'experimental') {
-                unless ($latest_entry->Changes
-                    =~ /\bto\s+['"‘“]?(?:unstable|sid)['"’”]?\b/im) {
-                    $self->hint('experimental-to-unstable-without-comment');
-                }
-            }
+
+            $self->pointed_hint('experimental-to-unstable-without-comment',
+                $latest_pointer)
+              if $latest_dist eq 'unstable'
+              && $previous_dist eq 'experimental'
+              && $latest_entry->Changes
+              !~ m{ \b to \s+ ['"\N{LEFT SINGLE QUOTATION MARK}\N{LEFT DOUBLE QUOTATION MARK}]? (?:unstable|sid) ['"\N{RIGHT SINGLE QUOTATION MARK}\N{RIGHT DOUBLE QUOTATION MARK}]? \b }imx;
 
             my $changes = $group->changes;
             if ($changes) {
@@ -653,12 +701,14 @@ sub binary {
                 my %codename;
                 $codename{'unstable'} = 'sid';
                 my @normalized
-                  = map { $codename{$_} // $_ }($latest_dist, $changes_dist);
+                  = uniq map { $codename{$_} // $_ }
+                  ($latest_dist, $changes_dist);
 
-                $self->hint(
+                $self->pointed_hint(
                     'changelog-distribution-does-not-match-changes-file',
-                    "($latest_dist != $changes_dist)")
-                  unless scalar(uniq @normalized) == 1;
+                    $latest_pointer,$latest_dist,
+                    $NOT_EQUALS, $changes_dist
+                )unless @normalized == 1;
             }
 
         }
@@ -668,23 +718,42 @@ sub binary {
         my $changes = $latest_entry->Changes || $EMPTY;
 
         if (@entries == 1) {
-            if ($latest_entry->Version and $latest_entry->Version =~ /-1$/) {
-                $self->hint('initial-upload-closes-no-bugs')
+
+            if ($latest_entry->Version && $latest_entry->Version =~ /-1$/) {
+                $self->pointed_hint('initial-upload-closes-no-bugs',
+                    $latest_pointer)
                   unless @{ $latest_entry->Closes };
-                $self->hint('new-package-uses-date-based-version-number',
+
+                $self->pointed_hint(
+                    'new-package-uses-date-based-version-number',
+                    $latest_pointer,
                     $latest_entry->Version,
-                    '(vs. 0~' . $latest_entry->Version .')')
-                  if $latest_entry->Version =~ m/^\d{8}/;
+                    '(better: 0~' . $latest_entry->Version .')'
+                )if $latest_entry->Version =~ m/^\d{8}/;
             }
-            $self->hint('changelog-is-dh_make-template')
+
+            $self->pointed_hint('changelog-is-dh_make-template',
+                $latest_pointer)
               if $changes
               =~ /(?:#?\s*)(?:\d|n)+ is the bug number of your ITP/i;
         }
+
         while ($changes =~ /(closes[\s;]*(?:bug)?\#?\s?\d{6,})[^\w]/ig) {
-            $self->hint('possible-missing-colon-in-closes', $1) if $1;
+
+            my $closes = $1;
+
+            $self->pointed_hint('possible-missing-colon-in-closes',
+                $latest_pointer, $closes)
+              if length $closes;
         }
+
         if ($changes =~ m/(TEMP-\d{7}-[0-9a-fA-F]{6})/) {
-            $self->hint('changelog-references-temp-security-identifier', $1);
+
+            my $temporary_cve = $1;
+
+            $self->pointed_hint(
+                'changelog-references-temp-security-identifier',
+                $latest_pointer, $temporary_cve);
         }
 
         # check for bad intended distribution
@@ -693,74 +762,108 @@ sub binary {
                             (?'intended'testing|unstable|experimental|sid)}xi
         ){
             my $intended = lc($+{intended});
-            if($intended eq 'sid') {
-                $intended = 'unstable';
-            }
+
+            $intended = 'unstable'
+              if $intended eq 'sid';
+
             my $uploaded = $latest_entry->Distribution;
-            unless ($uploaded eq 'UNRELEASED') {
-                unless($uploaded eq $intended) {
-                    $self->hint('bad-intended-distribution',
-                        "intended to $intended but uploaded to $uploaded");
-                }
-            }
+
+            $self->pointed_hint('bad-intended-distribution', $latest_pointer,
+                "intended for $intended but uploaded to $uploaded")
+              if $uploaded ne $intended
+              && $uploaded ne 'UNRELEASED';
         }
 
-        if($changes =~ /Close:\s+(\#\d+)/xi) {
-            $self->hint('misspelled-closes-bug',$1);
+        if ($changes =~ m{ (Close: \s+ [#] \d+) }xi) {
+
+            my $statement = $1;
+
+            $self->pointed_hint('misspelled-closes-bug', $latest_pointer,
+                $statement);
         }
 
         my $changesempty = $changes;
         $changesempty =~ s/\W//gms;
-        if (length($changesempty)==0) {
-            $self->hint('changelog-empty-entry')
-              unless $latest_entry->Distribution eq 'UNRELEASED';
-        }
+
+        $self->pointed_hint('changelog-empty-entry', $latest_pointer)
+          if !length $changesempty
+          && $latest_entry->Distribution ne 'UNRELEASED';
 
         # before bug 50004 bts removed bug instead of archiving
         for my $bug (@{$latest_entry->Closes}) {
-            $self->hint('improbable-bug-number-in-closes', $bug)
+
+            $self->pointed_hint('improbable-bug-number-in-closes',
+                $latest_pointer, $bug)
               if $bug < $FIRST_ARCHIVED_BUG_NUMBER
               || $bug >= $OUT_OF_REACH_BUG_NUMBER;
         }
 
         # Compare against NEWS.Debian if available.
-        if ($news and $news->Version) {
-            if ($latest_entry->Version eq $news->Version) {
-                for my $field (qw/Distribution Urgency/) {
-                    if ($latest_entry->$field ne $news->$field) {
-                        $self->hint('changelog-news-debian-mismatch',
-                            $field,
-                            $latest_entry->$field . ' != ' . $news->$field);
-                    }
-                }
-            }
-            unless (exists $allversions{$news->Version}) {
-                $self->hint('debian-news-entry-has-unknown-version',
-                    $news->Version);
-            }
+        for my $field (qw/Distribution Urgency/) {
+
+            $self->pointed_hint(
+                'changelog-news-debian-mismatch',
+                $news_item->pointer($news->position),
+                $field,
+                $latest_entry->$field,
+                $NOT_EQUALS,
+                $news->$field
+              )
+              if defined $news
+              && length $news->Version
+              && $news->Version eq $latest_entry->Version
+              && $news->$field ne $latest_entry->$field;
         }
 
+        $self->pointed_hint(
+            'debian-news-entry-has-unknown-version',
+            $news_item->pointer($news->position),
+            $news->Version
+          )
+          if defined $news
+          && length $news->Version
+          && !exists $allversions{$news->Version};
+
         # Parse::DebianChangelog adds an additional space to the
         # beginning of each line, so we have to adjust for that in the
         # length check.
         my @lines = split(/\n/, $changes);
-        for my $i (0 .. $#lines) {
-            my $line = $i + $chloff;
-            $self->hint('debian-changelog-line-too-short', $1, "(line $line)")
-              if $lines[$i] =~ /^   [*]\s(.{1,5})$/ and $1 !~ /:$/;
 
-            $self->hint('debian-changelog-line-too-long', "line $line")
-              if length $lines[$i] >= $MAXIMUM_WIDTH
-              && $lines[$i] !~ /^ [\s.o*+-]* (?: [Ss]ee:?\s+ )? \S+ $/msx;
+        # real start
+        my $position = $real_start;
+        for my $line (@lines) {
+
+            my $pointer = $changelog_item->pointer($position);
+
+            if ($line =~ /^   [*]\s(.{1,5})$/) {
+
+                my $excerpt = $1;
+
+                $self->pointed_hint('debian-changelog-line-too-short',
+                    $pointer, $excerpt)
+                  unless $1 =~ /:$/;
+            }
+
+            $self->pointed_hint('debian-changelog-line-too-long', $pointer)
+              if length $line >= $MAXIMUM_WIDTH
+              && $line !~ /^ [\s.o*+-]* (?: [Ss]ee:?\s+ )? \S+ $/msx;
+
+        } continue {
+            ++$position;
         }
 
         # Strip out all lines that contain the word spelling to avoid false
         # positives on changelog entries for spelling fixes.
         $changes =~ s/^.*(?:spelling|typo).*\n//gm;
+
         check_spelling(
-            $self->profile,$changes,
+            $self->data,
+            $changes,
             $group->spelling_exceptions,
-            $self->spelling_tag_emitter('spelling-error-in-changelog'));
+            $self->spelling_tag_emitter(
+                'spelling-error-in-changelog', $changelog_item
+            )
+        );
     }
 
     return;
@@ -771,40 +874,70 @@ sub binary {
 # the first line of text in the first entry.
 #
 sub check_dch {
-    my ($self, $path) = @_;
+    my ($self) = @_;
+
+    my $unresolved = $self->processable->changelog_item;
+
+    # stop for dangling symbolic link
+    my $item = $unresolved->resolve_path;
+    return 0
+      unless defined $item;
+
+    # return empty changelog
+    return 0
+      unless $item->is_file && $item->is_open_ok;
 
     # emacs only looks at the last "local variables" in a file, and only at
     # one within 3000 chars of EOF and on the last page (^L), but that's a bit
     # pesky to replicate.  Demanding a match of $prefix and $suffix ought to
     # be enough to avoid false positives.
 
-    my ($prefix, $suffix);
-    my $lineno = 0;
-    my ($estart, $tstart) = (0, 0);
+    my $contents;
+    if ($item->basename =~ m{ [.]gz $}x) {
 
-    open(my $fd, '<:utf8_strict', $path)
-      or die encode_utf8('Cannot open ' . $path);
+        my $bytes = safe_qx('gunzip', '-c', $item->unpacked_path);
 
-    while (my $line = <$fd>) {
+        return 0
+          unless valid_utf8($bytes);
 
-        unless ($tstart) {
-            $lineno++;
+        $contents = decode_utf8($bytes);
 
-            $estart = 1
-              if $line =~ /^\S/;
+    } else {
 
-            $tstart = 1
-              if $line =~ /^\s+\S/;
-        }
+        # empty unless valis UTF-8
+        $contents = $item->decoded_utf8;
+    }
+
+    my @lines = split(m{\n}, $contents);
+
+    my $prefix;
+    my $suffix;
+    my $real_start = 0;
+
+    my $saw_tab_lead = 0;
+
+    my $position = 1;
+    for my $line (@lines) {
+
+        ++$real_start
+          unless $saw_tab_lead;
+
+        $saw_tab_lead = 1
+          if $line =~ /^\s+\S/;
+
+        my $pointer = $item->pointer($position);
 
         if (
-               $line =~ /closes:\s*(((?:bug)?\#?\s?\d*)[[:alpha:]]\w*)/i
-            || $line =~ m{closes:\s*(?:bug)?\#?\s?\d+
-              (?:,\s*(?:bug)?\#?\s?\d+)*
-              (?:,\s*(((?:bug)?\#?\s?\d*)[[:alpha:]]\w*))}ix
+            $line
+            =~ m{ closes: \s* (( (?:bug)? [#]? \s? \d*) [[:alpha:]] \w*) }ix
+            || $line =~ m{ closes: \s* (?:bug)? [#]? \s? \d+
+              (?: , \s* (?:bug)? [#]? \s? \d+ )*
+              (?: , \s* (( (?:bug)? [#]? \s? \d* ) [[:alpha:]] \w*)) }ix
         ) {
-            $self->hint('wrong-bug-number-in-closes', $1,
-                "in the installed changelog (line $.)")
+
+            my $bug = $1;
+
+            $self->pointed_hint('wrong-bug-number-in-closes', $pointer, $bug)
               if length $2;
         }
 
@@ -814,15 +947,18 @@ sub check_dch {
         }
 
         # emacs allows whitespace between prefix and variable, hence \s*
-        $self->hint(
-            'debian-changelog-file-contains-obsolete-user-emacs-settings')
+        $self->pointed_hint(
+            'debian-changelog-file-contains-obsolete-user-emacs-settings',
+            $pointer)
           if defined $prefix
           && defined $suffix
           && $line =~ /^\Q$prefix\E\s*add-log-mailing-address:.*\Q$suffix\E$/;
-    }
-    close($fd);
 
-    return $lineno;
+    } continue {
+        ++$position;
+    }
+
+    return $real_start;
 }
 
 1;
