@@ -1,6 +1,6 @@
 # desktop/gnome -- lintian check script -*- perl -*-
 
-# Copyright © 1998 Christian Schwarz and Richard Braakman
+# Copyright (C) 1998 Christian Schwarz and Richard Braakman
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, you can find it on the World Wide
-# Web at http://www.gnu.org/copyleft/gpl.html, or write to the Free
+# Web at https://www.gnu.org/copyleft/gpl.html, or write to the Free
 # Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 # MA 02110-1301, USA.
 
@@ -30,11 +30,12 @@ use namespace::clean;
 with 'Lintian::Check';
 
 sub visit_installed_files {
-    my ($self, $file) = @_;
+    my ($self, $item) = @_;
 
     # /etc/gconf/schemas
-    $self->hint('package-installs-into-etc-gconf-schemas', $file->name)
-      if $file->name =~ m{^etc/gconf/schemas/\S};
+    $self->pointed_hint('package-installs-into-etc-gconf-schemas',
+        $item->pointer)
+      if $item->name =~ m{^etc/gconf/schemas/\S};
 
     return;
 }
