@@ -1,6 +1,6 @@
 # images -- lintian check script -*- perl -*-
 
-# Copyright © 1998 Christian Schwarz and Richard Braakman
+# Copyright (C) 1998 Christian Schwarz and Richard Braakman
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, you can find it on the World Wide
-# Web at http://www.gnu.org/copyleft/gpl.html, or write to the Free
+# Web at https://www.gnu.org/copyleft/gpl.html, or write to the Free
 # Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 # MA 02110-1301, USA.
 
@@ -30,12 +30,12 @@ use namespace::clean;
 with 'Lintian::Check';
 
 sub visit_installed_files {
-    my ($self, $file) = @_;
+    my ($self, $item) = @_;
 
-    $self->hint('image-file-in-usr-lib', $file->name)
-      if $file->name =~ m{^usr/lib/}
-      && $file->name =~ m{\.(?:bmp|gif|jpe?g|png|tiff|x[pb]m)$}
-      && !length $file->link;
+    $self->pointed_hint('image-file-in-usr-lib', $item->pointer)
+      if $item->name =~ m{^usr/lib/}
+      && $item->name =~ m{\.(?:bmp|gif|jpe?g|png|tiff|x[pb]m)$}
+      && !length $item->link;
 
     return;
 }
