@@ -1,9 +1,9 @@
 # scripts -- lintian check script -*- perl -*-
 #
-# Copyright © 1998 Richard Braakman
-# Copyright © 2002 Josip Rodin
-# Copyright © 2016-2019 Chris Lamb <lamby@debian.org>
-# Copyright © 2021 Felix Lechner
+# Copyright (C) 1998 Richard Braakman
+# Copyright (C) 2002 Josip Rodin
+# Copyright (C) 2016-2019 Chris Lamb <lamby@debian.org>
+# Copyright (C) 2021 Felix Lechner
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, you can find it on the World Wide
-# Web at http://www.gnu.org/copyleft/gpl.html, or write to the Free
+# Web at https://www.gnu.org/copyleft/gpl.html, or write to the Free
 # Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 # MA 02110-1301, USA.
 
@@ -33,17 +33,12 @@ use List::SomeUtils qw(any none);
 use Unicode::UTF8 qw(encode_utf8);
 
 use Lintian::IPC::Run3 qw(safe_qx);
-use Lintian::Pointer::Item;
 use Lintian::Relation;
 
-use Moo;
-use namespace::clean;
-
-with 'Lintian::Check';
-
 const my $EMPTY => q{};
 const my $SPACE => q{ };
 const my $SLASH => q{/};
+const my $AT_SIGN => q{@};
 const my $ASTERISK => q{*};
 const my $DOT => q{.};
 const my $DOUBLE_QUOTE => q{"};
@@ -53,6 +48,11 @@ const my $BAD_MAINTAINER_COMMAND_FIELDS => 5;
 const my $UNVERSIONED_INTERPRETER_FIELDS => 2;
 const my $VERSIONED_INTERPRETER_FIELDS => 5;
 
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
+
 # This is a map of all known interpreters.  The key is the interpreter
 # name (the binary invoked on the #! line).  The value is an anonymous
 # array of two elements.  The first argument is the path on a Debian
@@ -68,25 +68,29 @@ has INTERPRETERS => (
     default => sub {
         my ($self) = @_;
 
-        my $unversioned = $self->profile->load_data(
-            'scripts/interpreters',
-            qr/ \s* => \s* /msx,
-            sub {
-                my ($interpreter, $remainder) = @_;
+        my %unversioned;
 
-                my ($folder, $prerequisites)= split(/ \s* , \s* /msx,
-                    $remainder, $UNVERSIONED_INTERPRETER_FIELDS);
+        my $data
+          = $self->data->load('scripts/interpreters',qr/ \s* => \s* /msx);
 
-                $prerequisites //= $EMPTY;
+        for my $interpreter ($data->all) {
 
-                return {
-                    folder => $folder,
-                    prerequisites => $prerequisites
-                };
-            });
+            my $remainder = $data->value($interpreter);
 
-        return $unversioned;
-    });
+            my ($folder, $prerequisites)= split(/ \s* , \s* /msx,
+                $remainder, $UNVERSIONED_INTERPRETER_FIELDS);
+
+            $prerequisites //= $EMPTY;
+
+            $unversioned{$interpreter} = {
+                folder => $folder,
+                prerequisites => $prerequisites
+            };
+        }
+
+        return \%unversioned;
+    }
+);
 
 # The more complex case of interpreters that may have a version number.
 #
@@ -123,39 +127,43 @@ has VERSIONED_INTERPRETERS => (
     default => sub {
         my ($self) = @_;
 
-        my $versioned = $self->profile->load_data(
-            'scripts/versioned-interpreters',
-            qr/ \s* => \s* /msx,
-            sub {
-                my ($interpreter, $remainder) = @_;
+        my %versioned;
 
-                my ($folder, $regex, $template, $version_list, $prerequisites)
-                  = split(/ \s* , \s* /msx,
-                    $remainder, $VERSIONED_INTERPRETER_FIELDS);
+        my $data = $self->data->load('scripts/versioned-interpreters',
+            qr/ \s* => \s* /msx);
 
-                my @versions = split(/ \s+ /msx, $version_list);
-                $prerequisites //= $EMPTY;
+        for my $interpreter ($data->all) {
 
-                if ($prerequisites eq '@SKIP_UNVERSIONED@') {
-                    $prerequisites = undef;
+            my $remainder = $data->value($interpreter);
 
-                } elsif ($prerequisites =~ / @ /msx) {
-                    die encode_utf8(
+            my ($folder, $pattern, $template, $version_list, $prerequisites)
+              = split(/ \s* , \s* /msx,
+                $remainder, $VERSIONED_INTERPRETER_FIELDS);
+
+            my @versions = split(/ \s+ /msx, $version_list);
+            $prerequisites //= $EMPTY;
+
+            if ($prerequisites eq $AT_SIGN . 'SKIP_UNVERSIONED' . $AT_SIGN) {
+                $prerequisites = undef;
+
+            } elsif ($prerequisites =~ / @ /msx) {
+                die encode_utf8(
 "Unknown magic value $prerequisites for versioned interpreter $interpreter"
-                    );
-                }
+                );
+            }
 
-                return {
-                    folder => $folder,
-                    prerequisites => $prerequisites,
-                    regex => qr/^$regex$/,
-                    template => $template,
-                    versions => \@versions
-                };
-            });
+            $versioned{$interpreter} = {
+                folder => $folder,
+                prerequisites => $prerequisites,
+                regex => qr/^$pattern$/,
+                template => $template,
+                versions => \@versions
+            };
+        }
 
-        return $versioned;
-    });
+        return \%versioned;
+    }
+);
 
 # When detecting commands inside shell scripts, use this regex to match the
 # beginning of the command rather than checking whether the command is at the
@@ -171,44 +179,50 @@ has BAD_MAINTAINER_COMMANDS => (
     default => sub {
         my ($self) = @_;
 
-        return $self->profile->load_data(
-            'scripts/maintainer-script-bad-command',
-            qr/\s*\~\~/,
-            sub {
-                my ($in_cat,$in_auto,$package_include_pattern,
-                    $script_include_pattern,$command_pattern)
-                  = split(/ \s* ~~ /msx, $_[1],$BAD_MAINTAINER_COMMAND_FIELDS);
+        my %forbidden;
 
-                die encode_utf8(
-                    "Syntax error in scripts/maintainer-script-bad-command: $."
-                  )
-                  if any { !defined }
-                (
-                    $in_cat,$in_auto,$package_include_pattern,
-                    $script_include_pattern,$command_pattern
-                );
+        my $data = $self->data->load('scripts/maintainer-script-bad-command',
+            qr/\s*\~\~/);
 
-                # trim both ends
-                $in_cat =~ s/^\s+|\s+$//g;
-                $in_auto =~ s/^\s+|\s+$//g;
-                $package_include_pattern =~ s/^\s+|\s+$//g;
-                $script_include_pattern =~ s/^\s+|\s+$//g;
+        for my $key ($data->all) {
 
-                $package_include_pattern ||= '\a\Z';
+            my $value = $data->value($key);
 
-                $script_include_pattern ||= $DOT . $ASTERISK;
+            my ($in_cat,$in_auto,$package_include_pattern,
+                $script_include_pattern,$command_pattern)
+              = split(/ \s* ~~ /msx, $value,$BAD_MAINTAINER_COMMAND_FIELDS);
 
-                $command_pattern=~ s/\$[{]LEADING_PATTERN[}]/$LEADING_PATTERN/;
+            die encode_utf8(
+                "Syntax error in scripts/maintainer-script-bad-command: $.")
+              if any { !defined }(
+                $in_cat,$in_auto,$package_include_pattern,
+                $script_include_pattern,$command_pattern
+              );
 
-                return {
-                    ignore_automatic_sections => !!$in_auto,
-                    in_cat_string => !!$in_cat,
-                    package_exclude_regex => qr/$package_include_pattern/x,
-                    script_include_regex => qr/$script_include_pattern/x,
-                    command_pattern => $command_pattern,
-                };
-            });
-    });
+            # trim both ends
+            $in_cat =~ s/^\s+|\s+$//g;
+            $in_auto =~ s/^\s+|\s+$//g;
+            $package_include_pattern =~ s/^\s+|\s+$//g;
+            $script_include_pattern =~ s/^\s+|\s+$//g;
+
+            $package_include_pattern ||= '\a\Z';
+
+            $script_include_pattern ||= $DOT . $ASTERISK;
+
+            $command_pattern=~ s/\$[{]LEADING_PATTERN[}]/$LEADING_PATTERN/;
+
+            $forbidden{$key} = {
+                ignore_automatic_sections => !!$in_auto,
+                in_cat_string => !!$in_cat,
+                package_exclude_regex => qr/$package_include_pattern/x,
+                script_include_regex => qr/$script_include_pattern/x,
+                command_pattern => $command_pattern,
+            };
+        }
+
+        return \%forbidden;
+    }
+);
 
 # Appearance of one of these regexes in a maintainer script means that there
 # must be a dependency (or pre-dependency) on the given package.  The tag
@@ -238,7 +252,8 @@ has all_prerequisites => (
             $self->processable->name);
 
         return $all_prerequisites;
-    });
+    }
+);
 
 has strong_prerequisites => (
     is => 'rw',
@@ -249,7 +264,8 @@ has strong_prerequisites => (
         my $strong_prerequisites = $self->processable->relation('strong');
 
         return $strong_prerequisites;
-    });
+    }
+);
 
 sub visit_installed_files {
     my ($self, $item) = @_;
@@ -286,14 +302,11 @@ sub visit_installed_files {
 
     my $is_absolute = ($item->interpreter =~ m{^/} || $item->calls_env);
 
-    my $pointer = Lintian::Pointer::Item->new;
-    $pointer->item($item);
-
     # As a special-exception, Policy 10.4 states that Perl scripts must use
     # /usr/bin/perl directly and not via /usr/bin/env, etc.
     $self->pointed_hint(
         'incorrect-path-for-interpreter',
-        $pointer,'/usr/bin/env perl',
+        $item->pointer,'/usr/bin/env perl',
         $NOT_EQUAL, '/usr/bin/perl'
       )
       if $item->calls_env
@@ -302,7 +315,7 @@ sub visit_installed_files {
 
     $self->pointed_hint(
         'example-incorrect-path-for-interpreter',
-        $pointer,'/usr/bin/env perl',
+        $item->pointer,'/usr/bin/env perl',
         $NOT_EQUAL, '/usr/bin/perl'
       )
       if $item->calls_env
@@ -337,29 +350,30 @@ sub visit_installed_files {
 
     if ($item->interpreter eq $EMPTY) {
 
-        $self->pointed_hint('script-without-interpreter', $pointer)
+        $self->pointed_hint('script-without-interpreter', $item->pointer)
           if $item->name !~ m{^usr/share/doc/[^/]+/examples/};
 
-        $self->pointed_hint('example-script-without-interpreter', $pointer)
+        $self->pointed_hint('example-script-without-interpreter',
+            $item->pointer)
           if $item->name =~ m{^usr/share/doc/[^/]+/examples/};
 
         return;
     }
 
     # Either they use an absolute path or they use '/usr/bin/env interp'.
-    $self->pointed_hint('interpreter-not-absolute', $pointer,
+    $self->pointed_hint('interpreter-not-absolute', $item->pointer,
         $item->interpreter)
       if !$is_absolute
       && $item->name !~ m{^usr/share/doc/[^/]+/examples/};
 
     $self->pointed_hint('example-interpreter-not-absolute',
-        $pointer,$item->interpreter)
+        $item->pointer,$item->interpreter)
       if !$is_absolute
       && $item->name =~ m{^usr/share/doc/[^/]+/examples/};
 
     my $bash_completion_regex= qr{^usr/share/bash-completion/completions/.*};
 
-    $self->pointed_hint('script-not-executable', $pointer)
+    $self->pointed_hint('script-not-executable', $item->pointer)
       if (!$item->is_file || !$item->is_executable)
       && $item->name !~ m{^usr/(?:lib|share)/.*\.pm}
       && $item->name !~ m{^usr/(?:lib|share)/.*\.py}
@@ -382,16 +396,16 @@ sub visit_installed_files {
     # check $INTERPRETERS and %versioned_interpreters.  If not
     # found there, see if it ends in a version number and the base
     # is found in $VERSIONED_INTERPRETERS
-    my $interpreter_data = $self->INTERPRETERS->value($basename);
+    my $interpreter_data = $self->INTERPRETERS->{$basename};
 
     my $versioned = 0;
     unless (defined $interpreter_data) {
 
-        $interpreter_data = $self->VERSIONED_INTERPRETERS->value($basename);
+        $interpreter_data = $self->VERSIONED_INTERPRETERS->{$basename};
 
         if (!defined $interpreter_data && $basename =~ /^(.*[^\d.-])-?[\d.]+$/)
         {
-            $interpreter_data = $self->VERSIONED_INTERPRETERS->value($1);
+            $interpreter_data = $self->VERSIONED_INTERPRETERS->{$1};
             undef $interpreter_data
               unless $interpreter_data
               && $basename =~ /$interpreter_data->{regex}/;
@@ -406,28 +420,29 @@ sub visit_installed_files {
 
         my @context = ($item->interpreter, $NOT_EQUAL, $expected);
 
-        $self->pointed_hint('wrong-path-for-interpreter', $pointer, @context)
+        $self->pointed_hint('wrong-path-for-interpreter', $item->pointer,
+            @context)
           if $item->interpreter ne $expected
           && !$item->calls_env
           && $expected ne '/usr/bin/env perl'
           && $item->name !~ m{^usr/share/doc/[^/]+/examples/};
 
         $self->pointed_hint('example-wrong-path-for-interpreter',
-            $pointer, @context)
+            $item->pointer, @context)
           if $item->interpreter ne $expected
           && !$item->calls_env
           && $expected ne '/usr/bin/env perl'
           && $item->name =~ m{^usr/share/doc/[^/]+/examples/};
 
         $self->pointed_hint('incorrect-path-for-interpreter',
-            $pointer, @context)
+            $item->pointer, @context)
           if $item->interpreter ne $expected
           && !$item->calls_env
           && $expected eq '/usr/bin/env perl'
           && $item->name !~ m{^usr/share/doc/[^/]+/examples/};
 
         $self->pointed_hint('example-incorrect-path-for-interpreter',
-            $pointer, @context)
+            $item->pointer, @context)
           if $item->interpreter ne $expected
           && !$item->calls_env
           && $expected eq '/usr/bin/env perl'
@@ -435,49 +450,50 @@ sub visit_installed_files {
 
     } elsif ($item->interpreter =~ m{^/usr/local/}) {
 
-        $self->pointed_hint('interpreter-in-usr-local', $pointer,
+        $self->pointed_hint('interpreter-in-usr-local', $item->pointer,
             $item->interpreter)
           if $item->name !~ m{^usr/share/doc/[^/]+/examples/};
 
         $self->pointed_hint('example-interpreter-in-usr-local',
-            $pointer,$item->interpreter)
+            $item->pointer,$item->interpreter)
           if $item->name =~ m{^usr/share/doc/[^/]+/examples/};
 
     } elsif ($item->interpreter eq '/bin/env') {
 
-        $self->pointed_hint('script-uses-bin-env', $pointer,$item->interpreter)
+        $self->pointed_hint('script-uses-bin-env', $item->pointer,
+            $item->interpreter)
           if $item->name !~ m{^usr/share/doc/[^/]+/examples/};
 
-        $self->pointed_hint('example-script-uses-bin-env', $pointer,
+        $self->pointed_hint('example-script-uses-bin-env', $item->pointer,
             $item->interpreter)
           if $item->name =~ m{^usr/share/doc/[^/]+/examples/};
 
     } elsif ($item->interpreter eq 'nodejs') {
 
         $self->pointed_hint('script-uses-deprecated-nodejs-location',
-            $pointer,$item->interpreter)
+            $item->pointer,$item->interpreter)
           if $item->name !~ m{^usr/share/doc/[^/]+/examples/};
 
         $self->pointed_hint('example-script-uses-deprecated-nodejs-location',
-            $pointer,$item->interpreter)
+            $item->pointer,$item->interpreter)
           if $item->name =~ m{^usr/share/doc/[^/]+/examples/};
 
         # Check whether we have correct dependendies on nodejs regardless.
-        $interpreter_data = $self->INTERPRETERS->value('node');
+        $interpreter_data = $self->INTERPRETERS->{'node'};
 
     } elsif ($basename =~ /^php/) {
 
         $self->pointed_hint('php-script-with-unusual-interpreter',
-            $pointer,$item->interpreter)
+            $item->pointer,$item->interpreter)
           if $item->name !~ m{^usr/share/doc/[^/]+/examples/};
 
         $self->pointed_hint('example-php-script-with-unusual-interpreter',
-            $pointer, $item->interpreter)
+            $item->pointer, $item->interpreter)
           if $item->name =~ m{^usr/share/doc/[^/]+/examples/};
 
         # This allows us to still perform the dependencies checks
         # below even when an unusual interpreter has been found.
-        $interpreter_data = $self->INTERPRETERS->value('php');
+        $interpreter_data = $self->INTERPRETERS->{'php'};
 
     } else {
         my @private_interpreters;
@@ -498,11 +514,12 @@ sub visit_installed_files {
             push(@private_interpreters, grep { defined } @files);
         }
 
-        $self->pointed_hint('unusual-interpreter', $pointer,$item->interpreter)
+        $self->pointed_hint('unusual-interpreter', $item->pointer,
+            $item->interpreter)
           if (none { $_->is_file && $_->is_executable } @private_interpreters)
           && $item->name !~ m{^usr/share/doc/[^/]+/examples/};
 
-        $self->pointed_hint('example-unusual-interpreter', $pointer,
+        $self->pointed_hint('example-unusual-interpreter', $item->pointer,
             $item->interpreter)
           if (none { $_->is_file && $_->is_executable } @private_interpreters)
           && $item->name =~ m{^usr/share/doc/[^/]+/examples/};
@@ -529,17 +546,19 @@ sub visit_installed_files {
             if ($basename =~ /^php/) {
 
                 $self->pointed_hint('php-script-but-no-php-cli-dep',
-                    $pointer, $item->interpreter,
+                    $item->pointer, $item->interpreter,
                     "(does not satisfy $depends)");
 
             } elsif ($basename =~ /^(python\d|ruby|[mg]awk)$/) {
 
-                $self->pointed_hint((
-                    "$basename-script-but-no-$basename-dep",
-                    $pointer,
-                    $item->interpreter,
-                    "(does not satisfy $depends)"
-                ));
+                $self->pointed_hint(
+                    (
+                        "$basename-script-but-no-$basename-dep",
+                        $item->pointer,
+                        $item->interpreter,
+                        "(does not satisfy $depends)"
+                    )
+                );
 
             } elsif ($basename eq 'csh'
                 && $item->name =~ m{^etc/csh/login\.d/}){
@@ -563,12 +582,12 @@ sub visit_installed_files {
             } else {
 
                 $self->pointed_hint('missing-dep-for-interpreter',
-                    $pointer, $item->interpreter,
+                    $item->pointer, $item->interpreter,
                     "(does not satisfy $depends)");
             }
         }
 
-    } elsif ($self->VERSIONED_INTERPRETERS->recognizes($basename)) {
+    } elsif (exists $self->VERSIONED_INTERPRETERS->{$basename}) {
         my @versions = @{ $interpreter_data->{versions} };
 
         my @depends;
@@ -589,13 +608,13 @@ sub visit_installed_files {
 
                 $self->pointed_hint(
                     "$shell_name-script-but-no-$shell_name-dep",
-                    $pointer, $item->interpreter,
+                    $item->pointer, $item->interpreter,
                     "(does not satisfy $depends)");
 
             } else {
 
                 $self->pointed_hint('missing-dep-for-interpreter',
-                    $pointer, $item->interpreter,
+                    $item->pointer, $item->interpreter,
                     "(does not satisfy $depends)");
             }
         }
@@ -610,13 +629,13 @@ sub visit_installed_files {
             if ($basename =~ /^(python|ruby)/) {
 
                 $self->pointed_hint("$1-script-but-no-$1-dep",
-                    $pointer, $item->interpreter,
+                    $item->pointer, $item->interpreter,
                     "(does not satisfy $depends)");
 
             } else {
 
                 $self->pointed_hint('missing-dep-for-interpreter',
-                    $pointer, $item->interpreter,
+                    $item->pointer, $item->interpreter,
                     "(does not satisfy $depends)");
             }
         }
@@ -631,12 +650,9 @@ sub visit_control_files {
     return
       unless $item->is_maintainer_script;
 
-    my $rough_pointer = Lintian::Pointer::Item->new;
-    $rough_pointer->item($item);
-
     if ($item->is_elf) {
 
-        $self->pointed_hint('elf-maintainer-script', $rough_pointer);
+        $self->pointed_hint('elf-maintainer-script', $item->pointer);
         return;
     }
 
@@ -648,15 +664,15 @@ sub visit_control_files {
 
     if ($interpreter eq $EMPTY) {
 
-        $self->pointed_hint('script-without-interpreter', $rough_pointer);
+        $self->pointed_hint('script-without-interpreter', $item->pointer);
         return;
     }
 
     # tag for statistics
     $self->pointed_hint('maintainer-script-interpreter',
-        $rough_pointer, $interpreter);
+        $item->pointer, $interpreter);
 
-    $self->pointed_hint('interpreter-not-absolute', $rough_pointer,
+    $self->pointed_hint('interpreter-not-absolute', $item->pointer,
         $interpreter)
       unless $interpreter =~ m{^/};
 
@@ -664,13 +680,11 @@ sub visit_control_files {
 
     if ($interpreter =~ m{^/usr/local/}) {
         $self->pointed_hint('control-interpreter-in-usr-local',
-            $rough_pointer, $interpreter);
+            $item->pointer, $interpreter);
 
     } elsif ($basename eq 'sh' || $basename eq 'bash' || $basename eq 'perl') {
         my $expected
-          = ($self->INTERPRETERS->value($basename))->{folder}
-          . $SLASH
-          . $basename;
+          = $self->INTERPRETERS->{$basename}->{folder}. $SLASH. $basename;
 
         my $tag_name
           = ($expected eq '/usr/bin/env perl')
@@ -679,21 +693,21 @@ sub visit_control_files {
           : 'wrong-path-for-interpreter';
 
         $self->pointed_hint(
-            $tag_name, $rough_pointer,  $interpreter,
+            $tag_name, $item->pointer,  $interpreter,
             $NOT_EQUAL, $expected
         )unless $interpreter eq $expected;
 
     } elsif ($item->name eq 'config') {
         $self->pointed_hint('forbidden-config-interpreter',
-            $rough_pointer, $interpreter);
+            $item->pointer, $interpreter);
 
     } elsif ($item->name eq 'postrm') {
         $self->pointed_hint('forbidden-postrm-interpreter',
-            $rough_pointer, $interpreter);
+            $item->pointer, $interpreter);
 
-    } elsif ($self->INTERPRETERS->recognizes($basename)) {
+    } elsif (exists $self->INTERPRETERS->{$basename}) {
 
-        my $interpreter_data = $self->INTERPRETERS->value($basename);
+        my $interpreter_data = $self->INTERPRETERS->{$basename};
         my $expected = $interpreter_data->{folder} . $SLASH . $basename;
 
         my $tag_name
@@ -703,11 +717,11 @@ sub visit_control_files {
           : 'wrong-path-for-interpreter';
 
         $self->pointed_hint(
-            $tag_name, $rough_pointer, $interpreter,
+            $tag_name, $item->pointer, $interpreter,
             $NOT_EQUAL, $expected
         )unless $interpreter eq $expected;
 
-        $self->pointed_hint('unusual-control-interpreter', $rough_pointer,
+        $self->pointed_hint('unusual-control-interpreter', $item->pointer,
             $interpreter);
 
         # Interpreters used by preinst scripts must be in
@@ -722,7 +736,7 @@ sub visit_control_files {
 
                 $self->pointed_hint(
                     'control-interpreter-without-predepends',
-                    $rough_pointer,
+                    $item->pointer,
                     $interpreter,
                     '(does not satisfy ' . $depends->to_string . ')'
                   )
@@ -733,7 +747,7 @@ sub visit_control_files {
 
                 $self->pointed_hint(
                     'control-interpreter-without-depends',
-                    $rough_pointer,
+                    $item->pointer,
                     $interpreter,
                     '(does not satisfy ' . $depends->to_string . ')'
                   )
@@ -743,7 +757,7 @@ sub visit_control_files {
         }
 
     } else {
-        $self->pointed_hint('unknown-control-interpreter', $rough_pointer,
+        $self->pointed_hint('unknown-control-interpreter', $item->pointer,
             $interpreter);
 
         # no use doing further checks if it's not a known interpreter
@@ -770,9 +784,7 @@ sub visit_control_files {
     my $position = 1;
     while (my $line = <$fd>) {
 
-        my $pointer = Lintian::Pointer::Item->new;
-        $pointer->item($item);
-        $pointer->position($position);
+        my $pointer = $item->pointer($position);
 
         $saw_bange = 1
           if $position == 1
@@ -823,7 +835,7 @@ sub visit_control_files {
             $self->pointed_hint('udevadm-called-without-guard',$pointer)
               unless $saw_udevadm_guard
               || $line =~ m{\|\|}
-              || $self->strong_prerequisites->satisfies('udev');
+              || $self->strong_prerequisites->satisfies('udev:any');
         }
 
         if ($item->is_shell_script) {
@@ -858,13 +870,13 @@ sub visit_control_files {
                     $pointer)
                   if $line =~ m{>\s*/etc/inetd\.conf(?:\s|\Z)}
                   && !$self->processable->relation('Provides')
-                  ->satisfies('inet-superserver');
+                  ->satisfies('inet-superserver:any');
 
                 $self->pointed_hint('maintainer-script-modifies-inetd-conf',
                     $pointer)
                   if $line=~ m{^\s*(?:cp|mv)\s+(?:.*\s)?/etc/inetd\.conf\s*$}
                   && !$self->processable->relation('Provides')
-                  ->satisfies('inet-superserver');
+                  ->satisfies('inet-superserver:any');
 
                 # Check for running commands with a leading path.
                 #
@@ -975,10 +987,10 @@ sub visit_control_files {
 
     close $fd;
 
-    $self->pointed_hint('maintainer-script-without-set-e', $rough_pointer)
+    $self->pointed_hint('maintainer-script-without-set-e', $item->pointer)
       if $item->is_shell_script && !$saw_sete && $saw_bange;
 
-    $self->pointed_hint('maintainer-script-ignores-errors', $rough_pointer)
+    $self->pointed_hint('maintainer-script-ignores-errors', $item->pointer)
       if $item->is_shell_script && !$saw_sete && !$saw_bange;
 
     return;
@@ -989,9 +1001,9 @@ sub generic_check_bad_command {
         $in_automatic_section)
       = @_;
 
-    for my $tag_name ($self->BAD_MAINTAINER_COMMANDS->all) {
+    for my $tag_name (keys %{$self->BAD_MAINTAINER_COMMANDS}) {
 
-        my $command_data= $self->BAD_MAINTAINER_COMMANDS->value($tag_name);
+        my $command_data= $self->BAD_MAINTAINER_COMMANDS->{$tag_name};
 
         next
           if $in_automatic_section
@@ -1010,9 +1022,7 @@ sub generic_check_bad_command {
             # trim both ends
             $bad_command =~ s/^\s+|\s+$//g;
 
-            my $pointer = Lintian::Pointer::Item->new;
-            $pointer->item($script);
-            $pointer->position($position);
+            my $pointer = $script->pointer($position);
 
             $self->pointed_hint($tag_name, $pointer,
                 $DOUBLE_QUOTE . $bad_command . $DOUBLE_QUOTE)
